from collective.csv2dict import CSVImportError
from collective.csvsearcher.interfaces import IStorage
from collective.csvsearcher.reader import ApothecaryCSVReader
from operator import itemgetter
from zope.annotation.interfaces import IAnnotations
from zope.interface import implements


class Storage(object):
    implements(IStorage)

    ANNO_KEY = 'collective.csvsearcher'
    reader_class = ApothecaryCSVReader
    default_sort = 'sorteer_naam'

    def __init__(self, context):
        self.context = context

    def get_data(self):
        ann = IAnnotations(self.context)
        return ann.setdefault(self.ANNO_KEY + '.data', [])

    @property
    def field_names(self):
        ann = IAnnotations(self.context)
        return ann.setdefault(self.ANNO_KEY + '.field_names', [])

    @property
    def nrows(self):
        ann = IAnnotations(self.context)
        return ann.setdefault(self.ANNO_KEY + '.nrows', 0)

    @property
    def districts(self):
        ann = IAnnotations(self.context)
        return ann.setdefault(self.ANNO_KEY + '.districts', set())

    @property
    def naam_apotheek(self):
        ann = IAnnotations(self.context)
        return ann.setdefault(self.ANNO_KEY + '.naam_apotheek', set())

    @property
    def plaats_apotheek(self):
        ann = IAnnotations(self.context)
        return ann.setdefault(self.ANNO_KEY + '.plaats_apotheek', set())

    @property
    def plaats_apotheker(self):
        ann = IAnnotations(self.context)
        return ann.setdefault(self.ANNO_KEY + '.plaats_apotheker', set())

    @property
    def soort_lid(self):
        ann = IAnnotations(self.context)
        return ann.setdefault(self.ANNO_KEY + '.soort_lid', set())

    @property
    def sorteer_naam(self):
        ann = IAnnotations(self.context)
        return ann.setdefault(self.ANNO_KEY + '.sorteer_naam', set())

    def _get_unique_values(self, name, required=''):
        # Get unique values that are not empty and that contain the
        # required text.  By giving a required text, we can weed out
        # bad data, for example districts that do not have 'district'
        # in their text.  We compare with lowercase.
        required = required.lower()
        data = self.get_data()
        values = set([x.get(name) for x in data])
        values = filter(lambda v: v and required in v.lower(),
                        values)
        return sorted(values)

    def _reload(self):
        # First clear the helper data.
        ann = IAnnotations(self.context)
        ann[self.ANNO_KEY + '.districts'] = set()
        ann[self.ANNO_KEY + '.field_names'] = []
        ann[self.ANNO_KEY + '.naam_apotheek'] = set()
        ann[self.ANNO_KEY + '.plaats_apotheek'] = set()
        ann[self.ANNO_KEY + '.plaats_apotheker'] = set()
        ann[self.ANNO_KEY + '.soort_lid'] = set()
        ann[self.ANNO_KEY + '.sorteer_naam'] = set()

        # Now recalculate helper data.
        data = self.get_data()
        ann[self.ANNO_KEY + '.nrows'] = len(data)
        if not data:
            return
        first = data[0]
        # We might want to force a list of dictionaries as data.
        if hasattr(first, 'keys'):
            # Store list of field names.
            ann[self.ANNO_KEY + '.field_names'] = sorted(first.keys())
        # Sort the data according to the default sort key.
        if self.default_sort in self.field_names:
            ann[self.ANNO_KEY + '.data'] = sorted(
                data, key=itemgetter(self.default_sort))
        # Prepare vocabulary values.
        ann[self.ANNO_KEY + '.districts'] = self._get_unique_values(
            'district_apotheek', required='district')
        ann[self.ANNO_KEY + '.naam_apotheek'] = self._get_unique_values(
            'naam_apotheek')
        ann[self.ANNO_KEY + '.plaats_apotheek'] = self._get_unique_values(
            'plaats_apotheek')
        ann[self.ANNO_KEY + '.plaats_apotheker'] = self._get_unique_values(
            'plaats_apotheker')
        ann[self.ANNO_KEY + '.soort_lid'] = self._get_unique_values(
            'soort_lid', required='lid')
        ann[self.ANNO_KEY + '.sorteer_naam'] = self._get_unique_values(
            'sorteer_naam')

    def store(self, data):
        if not data:
            return False
        ann = IAnnotations(self.context)
        ann[self.ANNO_KEY + '.data'] = data
        self._reload()
        return True

    def store_csv(self, data, **kwargs):
        """Store csv data."""
        reader = self.reader_class(data.splitlines(), **kwargs)
        try:
            newdata = list(reader)
        except CSVImportError:
            newdata = []
        if not newdata:
            return False
        return self.store(newdata)

    def clear(self):
        ann = IAnnotations(self.context)
        ann[self.ANNO_KEY + '.data'] = []
        self._reload()

    def _parse_search_criteria(self, **kwargs):
        # We currently get unicode values from the form.  If not, then
        # we would need to call .decode('utf-8') on them to avoid a
        # unicode error.  Also, we may get a list of values in some of
        # the keyword arguments.
        opts = []
        for k, v in kwargs.items():
            if k not in self.field_names:
                continue
            if not v:
                continue
            if isinstance(v, basestring):
                v = v.lower()
            opts.append((k, v))
        return opts

    def search(self, **kwargs):
        results = []
        opts = self._parse_search_criteria(**kwargs)
        for item in self.get_data():
            # Assume a match, until proven otherwise.
            match = True
            for key, value in opts:
                if isinstance(value, basestring):
                    if value not in item.get(key).lower():
                        match = False
                        break
                elif isinstance(value, list):
                    # Currently we do not lowercase these.
                    if item.get(key) not in value:
                        match = False
                        break
                else:
                    # integer?
                    if value != item.get(key):
                        match = False
                        break

            if match:
                results.append(item)
        if kwargs.get('reverse') or kwargs.get('sort_order') == 'descending':
            reverse = True
        else:
            reverse = False
        sort_on = kwargs.get('sort_on')
        if sort_on and sort_on in self.field_names:
            # non-default sort_on, possibly reversed
            results = sorted(results, key=itemgetter(sort_on), reverse=reverse)
        elif reverse:
            # default sort_on, but reversed
            results.reverse()
        return results
