# -*- coding: utf-8 -*-
# Functional tests, using the testbrowser.

import transaction
import unittest
from collective.csvsearcher.interfaces import IStorage
from collective.csvsearcher.testing import BASIC_FUNCTIONAL_TESTING
from collective.csvsearcher.testing import get_file_upload
from plone.app.testing import TEST_USER_NAME, TEST_USER_PASSWORD
from plone.app.testing import SITE_OWNER_NAME, SITE_OWNER_PASSWORD
from plone.testing.z2 import Browser
from zExceptions import Unauthorized


class UploadTestCase(unittest.TestCase):
    layer = BASIC_FUNCTIONAL_TESTING

    def setUp(self):
        app = self.layer['app']
        self.browser = Browser(app)
        self.browser.handleErrors = False
        self.portal = self.layer['portal']

    def _browse_as_site_owner(self):
        self.browser.addHeader('Authorization', 'Basic %s:%s' % (
            SITE_OWNER_NAME, SITE_OWNER_PASSWORD,))

    def _browse_as_member(self):
        self.browser.addHeader('Authorization', 'Basic %s:%s' % (
            TEST_USER_NAME, TEST_USER_PASSWORD,))

    def test_no_member_upload(self):
        self._browse_as_member()
        self.assertRaises(Unauthorized, self.browser.open,
                          self.portal.absolute_url() + '/@@csvsearcher-upload')

    def test_no_upload(self):
        self._browse_as_site_owner()
        self.browser.open(self.portal.absolute_url() + '/@@csvsearcher-upload')
        self.assertTrue('Upload csv file' in self.browser.contents)
        self.assertTrue('Number of rows: 0.' in self.browser.contents)
        # Submitting the form without a csv file gives an error.
        self.browser.getControl('Upload').click()
        self.assertTrue('Required input is missing.' in self.browser.contents)

    def test_wrong_upload(self):
        self._browse_as_site_owner()
        self.browser.open(self.portal.absolute_url() + '/@@csvsearcher-upload')
        csv_control = self.browser.getControl(name='form.widgets.csvfile')
        csv_control.add_file(*get_file_upload('wrong.csv'))
        self.browser.getControl('Upload').click()
        self.assertTrue('CSV stored.' not in self.browser.contents)
        self.assertTrue(
            'No correct info found in CSV.' in self.browser.contents)
        self.assertTrue('Number of rows: 0.' in self.browser.contents)

    def test_non_strict_upload(self):
        # We can be less strict on wrong uploads if we want to.
        self._browse_as_site_owner()
        self.browser.open(self.portal.absolute_url() + '/@@csvsearcher-upload')
        csv_control = self.browser.getControl(name='form.widgets.csvfile')
        csv_control.add_file(*get_file_upload('wrong.csv'))
        self.browser.getControl('Strict').selected = False
        self.browser.getControl('Upload').click()
        self.assertTrue('CSV stored.' in self.browser.contents)
        self.assertTrue(
            'No correct info found in CSV.' not in self.browser.contents)
        self.assertTrue('Number of rows: 1.' in self.browser.contents)

    def test_small_upload(self):
        self._browse_as_site_owner()
        self.browser.open(self.portal.absolute_url() + '/@@csvsearcher-upload')
        csv_control = self.browser.getControl(name='form.widgets.csvfile')
        csv_control.add_file(*get_file_upload('iso8859.csv'))
        self.browser.getControl('Upload').click()
        self.assertTrue('CSV stored.' in self.browser.contents)
        self.assertTrue(
            'No correct info found in CSV.' not in self.browser.contents)
        self.assertTrue('Number of rows: 3.' in self.browser.contents)

    def test_wrong_upload_keeps_data(self):
        self._browse_as_site_owner()
        # First upload correct data.
        self.browser.open(self.portal.absolute_url() + '/@@csvsearcher-upload')
        csv_control = self.browser.getControl(name='form.widgets.csvfile')
        csv_control.add_file(*get_file_upload('iso8859.csv'))
        self.browser.getControl('Upload').click()
        self.assertTrue('Number of rows: 3.' in self.browser.contents)
        # Now upload wrong data, with a column too many.
        self.browser.open(self.portal.absolute_url() + '/@@csvsearcher-upload')
        csv_control = self.browser.getControl(name='form.widgets.csvfile')
        csv_control.add_file(*get_file_upload('wrong.csv'))
        self.browser.getControl('Upload').click()
        self.assertTrue('CSV stored.' not in self.browser.contents)
        self.assertTrue(
            'No correct info found in CSV.' in self.browser.contents)
        self.assertTrue('Number of rows: 3.' in self.browser.contents)

    def test_clear(self):
        self._browse_as_site_owner()
        # First upload data.
        self.browser.open(self.portal.absolute_url() + '/@@csvsearcher-upload')
        csv_control = self.browser.getControl(name='form.widgets.csvfile')
        csv_control.add_file(*get_file_upload('iso8859.csv'))
        self.browser.getControl('Upload').click()
        self.assertTrue('Number of rows: 3.' in self.browser.contents)
        # Now clear the data.
        self.browser.getControl('Remove all data').click()
        self.assertTrue('All data has been removed.' in self.browser.contents)
        self.assertTrue('Number of rows: 0.' in self.browser.contents)
        # You could select a csv and then clear the data anyway.  The
        # clear button wins.
        csv_control = self.browser.getControl(name='form.widgets.csvfile')
        csv_control.add_file(*get_file_upload('iso8859.csv'))
        self.browser.getControl('Remove all data').click()
        self.assertTrue('All data has been removed.' in self.browser.contents)
        self.assertTrue('CSV stored.' not in self.browser.contents)
        self.assertTrue('Number of rows: 0.' in self.browser.contents)


class SearchTestCase(unittest.TestCase):
    layer = BASIC_FUNCTIONAL_TESTING

    def setUp(self):
        app = self.layer['app']
        self.browser = Browser(app)
        self.browser.handleErrors = False
        self.portal = self.layer['portal']
        # Grant Member permission for searching.  We will want to do
        # that in rolemap.xml later.
        self.portal.manage_permission(
            'csvsearcher: Search', ['Member', 'Manager'])
        storage = IStorage(self.portal)
        result = storage.store_csv(get_file_upload('iso8859.csv')[0])
        self.assertEqual(result, True)
        transaction.commit()

    def _browse_as_member(self):
        self.browser.addHeader('Authorization', 'Basic %s:%s' % (
            TEST_USER_NAME, TEST_USER_PASSWORD,))

    def test_anonymous_no_view_search(self):
        self.assertRaises(Unauthorized, self.browser.open,
                          self.portal.absolute_url() + '/@@csvsearcher')

    def test_search_all(self):
        self._browse_as_member()
        self.browser.open(self.portal.absolute_url() + '/@@csvsearcher')
        form = self.browser.getForm(id='csvsearcher')
        form.submit(name='csvsearcher.buttons.search')
        self.assertTrue(self.browser.url.startswith(
            self.portal.absolute_url() + '/@@csvsearcher'))
        # Test that text from all three items is in the browser,
        # including some non-ascii characters.
        self.assertTrue('Rotterdam' in self.browser.contents)
        self.assertTrue('Belg' in self.browser.contents)
        self.assertTrue('Schäfer' in self.browser.contents)

    def test_search_single(self):
        self._browse_as_member()
        self.browser.open(self.portal.absolute_url() + '/@@csvsearcher')
        form = self.browser.getForm(id='csvsearcher')
        form.getControl(name='csvsearcher.widgets.sorteer_naam').value = 'Rees'
        form.submit(name='csvsearcher.buttons.search')
        self.assertTrue('Rotterdam' in self.browser.contents)
        self.assertTrue('Belg' not in self.browser.contents)

    def test_search_clear(self):
        self._browse_as_member()
        self.browser.open(self.portal.absolute_url() + '/@@csvsearcher')
        form = self.browser.getForm(id='csvsearcher')
        form.getControl(name='csvsearcher.widgets.sorteer_naam').value = 'Rees'
        form.submit(name='csvsearcher.buttons.clear')
        self.assertTrue('Rotterdam' not in self.browser.contents)

    def test_search_non_ascii(self):
        self._browse_as_member()
        self.browser.open(self.portal.absolute_url() + '/@@csvsearcher')
        # Simple accent
        form = self.browser.getForm(id='csvsearcher')
        form.getControl(
            name='csvsearcher.widgets.sorteer_naam').value = 'Schäfer'
        form.submit(name='csvsearcher.buttons.search')
        self.assertTrue('Tjökermeerdorp' in self.browser.contents)
        self.assertTrue('Belg' not in self.browser.contents)

    def test_search_utf8(self):
        storage = IStorage(self.portal)
        # Change the encoding on the fly.
        orig_encoding = storage.reader_class.encoding
        storage.reader_class.encoding = 'utf-8'
        try:
            result = storage.store_csv(get_file_upload('utf8.csv')[0])
        finally:
            # Restore the encoding
            storage.reader_class.encoding = orig_encoding
        self.assertEqual(result, True)
        transaction.commit()

        self._browse_as_member()
        self.browser.open(self.portal.absolute_url() + '/@@csvsearcher')
        # Chinese character
        form = self.browser.getForm(id='csvsearcher')
        form.getControl(name='csvsearcher.widgets.sorteer_naam').value = '汉'
        form.submit(name='csvsearcher.buttons.search')
        self.assertTrue('漢語' in self.browser.contents)
        self.assertTrue('Belg' not in self.browser.contents)

    def test_search_lowercase(self):
        self._browse_as_member()
        self.browser.open(self.portal.absolute_url() + '/@@csvsearcher')
        form = self.browser.getForm(id='csvsearcher')
        form.getControl(name='csvsearcher.widgets.sorteer_naam').value = 'rees'
        form.submit(name='csvsearcher.buttons.search')
        self.assertTrue('Rotterdam' in self.browser.contents)
        self.assertTrue('Belg' not in self.browser.contents)

    def test_search_city(self):
        self._browse_as_member()
        self.browser.open(self.portal.absolute_url() + '/@@csvsearcher')
        # zip code should not be there yet
        self.assertTrue('1000' not in self.browser.contents)

        # Search for correct work city
        form = self.browser.getForm(id='csvsearcher')
        form.getControl(
            name='csvsearcher.widgets.plaats_apotheek').value = 'Antwerpen'
        form.submit(name='csvsearcher.buttons.search')
        self.assertTrue('1000' in self.browser.contents)
        self.assertTrue('Belgweg' in self.browser.contents)

        # Search for correct living city
        form = self.browser.getForm(id='csvsearcher')
        form.submit(name='csvsearcher.buttons.clear')
        form = self.browser.getForm(id='csvsearcher')
        form.getControl(
            name='csvsearcher.widgets.plaats_apotheker').value = 'Dendermonde'
        form.submit(name='csvsearcher.buttons.search')
        self.assertTrue('1000' in self.browser.contents)
        self.assertTrue('Belgweg' in self.browser.contents)

        # Combine the correct searches
        form = self.browser.getForm(id='csvsearcher')
        form.submit(name='csvsearcher.buttons.clear')
        form = self.browser.getForm(id='csvsearcher')
        form.getControl(
            name='csvsearcher.widgets.plaats_apotheek').value = 'Antwerpen'
        form.getControl(
            name='csvsearcher.widgets.plaats_apotheker').value = 'Dendermonde'
        form.submit(name='csvsearcher.buttons.search')
        self.assertTrue('1000' in self.browser.contents)
        self.assertTrue('Belgweg' in self.browser.contents)

        # Search for wrong work city
        form = self.browser.getForm(id='csvsearcher')
        form.submit(name='csvsearcher.buttons.clear')
        form = self.browser.getForm(id='csvsearcher')
        form.getControl(
            name='csvsearcher.widgets.plaats_apotheek').value = 'Dendermonde'
        form.submit(name='csvsearcher.buttons.search')
        self.assertTrue('1000' not in self.browser.contents)
        self.assertTrue('Belgweg' not in self.browser.contents)

        # Search for wrong living city
        form = self.browser.getForm(id='csvsearcher')
        form.submit(name='csvsearcher.buttons.clear')
        form = self.browser.getForm(id='csvsearcher')
        form.getControl(
            name='csvsearcher.widgets.plaats_apotheker').value = 'Antwerpen'
        form.submit(name='csvsearcher.buttons.search')
        self.assertTrue('1000' not in self.browser.contents)
        self.assertTrue('Belgweg' not in self.browser.contents)

        # Combine the wrong searches
        form = self.browser.getForm(id='csvsearcher')
        form.submit(name='csvsearcher.buttons.clear')
        form = self.browser.getForm(id='csvsearcher')
        form.getControl(
            name='csvsearcher.widgets.plaats_apotheek').value = 'Dendermonde'
        form.getControl(
            name='csvsearcher.widgets.plaats_apotheker').value = 'Antwerpen'
        form.submit(name='csvsearcher.buttons.search')
        self.assertTrue('1000' not in self.browser.contents)
        self.assertTrue('Belgweg' not in self.browser.contents)

    def test_search_country(self):
        self._browse_as_member()
        self.browser.open(self.portal.absolute_url() + '/@@csvsearcher')
        # Search Belgium.
        form = self.browser.getForm(id='csvsearcher')
        form.getControl(
            name='csvsearcher.widgets.plaats_apotheek').value = 'Antwerpen'
        form.submit(name='csvsearcher.buttons.search')
        # Country should be there, as it is not the Netherlands.
        self.assertTrue('België' in self.browser.contents)
        self.assertTrue('Nederland' not in self.browser.contents)
        # Now try the Netherlands.
        form = self.browser.getForm(id='csvsearcher')
        form.getControl(
            name='csvsearcher.widgets.plaats_apotheek').value = 'Rotterdam'
        form.submit(name='csvsearcher.buttons.search')
        # country should not be there, as it is The Netherlands.
        self.assertTrue('België' not in self.browser.contents)
        self.assertTrue('Nederland' not in self.browser.contents)
