# -*- coding: utf-8 -*-
# Integration tests, with Plone Site, but without using the testbrowser.

import unittest
from collective.csvsearcher.interfaces import IStorage
from collective.csvsearcher.storage import Storage
from collective.csvsearcher.testing import BASIC_INTEGRATION_TESTING
from collective.csvsearcher.testing import get_file_upload
from zope.interface.verify import verifyClass, verifyObject


class StorageTestCase(unittest.TestCase):
    layer = BASIC_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

    def test_interface(self):
        # test the class
        self.assertTrue(IStorage.implementedBy, Storage)
        self.assertTrue(verifyClass(IStorage, Storage))
        # test an instance
        storage = Storage(self.portal)
        self.assertTrue(IStorage.providedBy, storage)
        self.assertTrue(verifyObject(IStorage, storage))

    def test_adapt(self):
        storage = IStorage(object(), None)
        self.assertTrue(storage is None)
        storage = IStorage(self.portal, None)
        self.assertTrue(storage is not None)

    def test_empty_storage(self):
        storage = IStorage(self.portal)
        self.assertEqual(storage.get_data(), [])
        self.assertEqual(storage.nrows, 0)

    def test_store(self):
        storage = IStorage(self.portal)
        storage.store([{str(x): str(x)} for x in range(10)])
        self.assertEqual(storage.nrows, 10)
        storage.store([{str(x): str(x)} for x in range(8)])
        self.assertEqual(storage.nrows, 8)

    def test_store_csv(self):
        storage = IStorage(self.portal)
        result = storage.store_csv(get_file_upload('iso8859.csv')[0])
        self.assertEqual(result, True)

    def test_clear(self):
        storage = IStorage(self.portal)
        storage.store([{str(x): str(x)} for x in range(10)])
        self.assertEqual(len(storage.get_data()), 10)
        # Storing an empty list is not allowed, because we do not want
        # to accidentally lose data.
        storage.store([])
        self.assertEqual(len(storage.get_data()), 10)
        # So we have a clear method.
        storage.clear()
        self.assertEqual(len(storage.get_data()), 0)

    def test_field_names(self):
        storage = IStorage(self.portal)
        self.assertEqual(storage.field_names, [])
        storage.store([{'one': 1, 'two': 2}])
        self.assertEqual(storage.field_names, ['one', 'two'])
        storage.clear()
        self.assertEqual(storage.field_names, [])

    def test_encoding(self):
        storage = IStorage(self.portal)
        self.assertEqual(storage.reader_class.encoding, 'iso8859')

    def test_search(self):
        storage = IStorage(self.portal)
        data = [
            {'sorteer_naam': 'rees', 'irc': 'maurits'},
            {'sorteer_naam': 'dijk', 'irc': 'fredvd'},
            {'sorteer_naam': 'ladage', 'irc': 'jladage'},
            {'sorteer_naam': 'reedijk', 'irc': 'pietje'},
        ]
        storage.store(data)
        self.assertEqual(storage.field_names, ['irc', 'sorteer_naam'])
        # Without arguments, we return all data.
        self.assertEqual(len(storage.search()), 4)
        # Search for rees:
        self.assertEqual(len(storage.search(sorteer_naam='rees')), 1)
        self.assertEqual(storage.search(sorteer_naam='rees')[0],
                         {'sorteer_naam': 'rees', 'irc': 'maurits'})
        # Search for ree:
        self.assertEqual(
            len(storage.search(sorteer_naam='ree')), 2)
        self.assertEqual(
            storage.search(sorteer_naam='ree')[0],
            {'sorteer_naam': 'reedijk', 'irc': 'pietje'})
        self.assertEqual(
            storage.search(sorteer_naam='ree')[1],
            {'sorteer_naam': 'rees', 'irc': 'maurits'})
        # Reverse search for ree:
        self.assertEqual(
            len(storage.search(sorteer_naam='ree', reverse=True)), 2)
        self.assertEqual(
            storage.search(sorteer_naam='ree', reverse=True)[1],
            {'sorteer_naam': 'reedijk', 'irc': 'pietje'})
        self.assertEqual(
            storage.search(sorteer_naam='ree', reverse=True)[0],
            {'sorteer_naam': 'rees', 'irc': 'maurits'})
        # Search for dijk:
        self.assertEqual(
            len(storage.search(sorteer_naam='dijk')), 2)
        self.assertEqual(
            storage.search(sorteer_naam='dijk')[0],
            {'sorteer_naam': 'dijk', 'irc': 'fredvd'})
        self.assertEqual(
            storage.search(sorteer_naam='dijk')[1],
            {'sorteer_naam': 'reedijk', 'irc': 'pietje'})
